/*
 * policy.c - CMDU policy for Easymesh TLVs
 *
 * Copyright (C) 2022 IOPSYS Software Solutions AB. All rights reserved.
 *
 * See LICENSE file for license related information.
 *
 */


#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>

#include "util.h"
#include "bufutil.h"
#include "1905_tlvs.h"
#include "cmdu.h"
#include "easymesh.h"
#include "map_module.h"

#define CMDU_TYPE_MAP_START	CMDU_1905_ACK
#define CMDU_TYPE_MAP_END	MAP_CMDU_TYPE_MAX

struct cmdu_tlv_policy {
	size_t num;
	struct tlv_policy *pol;
};

#include "r2.c"
#include "r4.c"


int map_cmdu_parse_tlvs(struct cmdu_buff *cmdu, struct tlv *tv[][16], int num_tv, int revision)
{
	struct cmdu_tlv_policy *pol = NULL;
	uint16_t type;
	int idx = 0;
	int i;


	if (!cmdu)
		return -1;

	type = cmdu_get_type(cmdu);

	if (type >= CMDU_TYPE_1905_START && type <= CMDU_TYPE_1905_END) {
		idx = type;
	} else if (type >= CMDU_TYPE_MAP_START && type <= CMDU_TYPE_MAP_END) {
		idx = type - CMDU_TYPE_MAP_START + CMDU_TYPE_1905_END + 1;
	} else
		return -1;

	switch (revision) {
	case 2:
		if (idx > ARRAY_SIZE(easymesh_policy_r2))
			return -1;

		pol = &easymesh_policy_r2[idx];
		break;
#if (EASYMESH_VERSION > 2)
	case 4:
		if (idx > ARRAY_SIZE(easymesh_policy_r4))
			return -1;

		pol = &easymesh_policy_r4[idx];
		break;
#endif
	default:
		return -1;
	}

	if (pol->pol == NULL)
		return -1;

	if (pol->num == 0)
		return 0;

	if (num_tv < pol->num) {
		fprintf(stderr, "%s: minimum %zu tv needed!\n", __func__, pol->num);
		return -1;
	}

	/* explicitly zero out the passed tlvs array before using */
	for (i = 0; i < num_tv; i++)
		memset(tv[i], 0, 16 * sizeof(struct tlv *));

	return cmdu_parse_tlvs(cmdu, tv, pol->pol, pol->num);
}
